"""
Utility functions for the CLI
"""
import board
import adafruit_ws2801


odata = board.D5
oclock = board.D6
numleds = 25
bright = 0.25

Leds = adafruit_ws2801.WS2801(
    oclock, odata, numleds, brightness=bright, auto_write=False
)


"""
Entrypoint for CLI.
"""
import click

from kubeleds.commands import get_cluster_nodes, get_namespaced_pods, set_leds,  init, show


@click.group(chain=True)
@click.pass_context
def cli(ctx):
    ctx.ensure_object(dict)


cli.add_command(init)
cli.add_command(show)
cli.add_command(get_cluster_nodes)
cli.add_command(get_namespaced_pods)
cli.add_command(set_leds)


if __name__ == '__main__':
    cli()



"""
Show command for the CLI.
"""
import click

from kubeleds import console


@click.command()
def show():
    """
    Generic sub-command to show a message.
    """
    console.print("Get started with your CLI in no time!")



"""
CLI initialization command.
"""
import click
from rich.prompt import Prompt

from kubeleds import console
from kubeleds.constants import WELCOME_MESSAGE


@click.command()
def init():
    """
    CLI Initialization demo.
    """
    console.print(WELCOME_MESSAGE)



"""Package setup"""
import setuptools


with open("README.md", "r") as f:
    long_description = f.read()

requirements = ["click", "rich<=7.1.0", "simple-term-menu", "requests", "RPI.GPIO", "adafruit-blinka", "adafruit-ws2801", "kubernetes"]


setuptools.setup(
    name="kubeleds",
    version="0.0.1",
    author="Robert Evans",
    description="monitor kubernetes clusters using adafruit neopixels",
    packages=setuptools.find_packages(
        exclude=["dist", "build", "*.egg-info", "tests"]
    ),
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=requirements,
    entry_points={
        "console_scripts": [
            "kubeledscli = kubeleds.app:cli"
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Operating System :: OS Independent",
        "Development Status :: 4 - Beta",
    ],
)



"""
Constants for the CLI.
"""
WELCOME_MESSAGE = "Welcome to [bold green]kubeleds!![/bold green] Your CLI is working!"

COLOR_OFF = (0, 0, 0)
COLOR_READY = (0, 255, 0)
COLOR_MEMORY_PRESSURE_GOOD = (0, 255, 255)
COLOR_DISK_PRESSURE_GOOD = (180, 0, 255)
COLOR_PID_PRESSURE_GOOD = (0, 0, 255)
COLOR_BAD = (255, 0, 0)

COLORS = {"Ready": (0, 255, 0), "MemoryPressure": (0, 255, 255), "DiskPressure": (180, 0, 255), "PIDPressure": (0, 0, 255)}
STATUS_CONDITIONS = ["Ready", "MemoryPressure", "DiskPressure", "PIDPressure"]


import click, json, random, time
from click.core import Context
from kubeleds import console
from kubeleds import utils
from kubernetes import client
from kubeleds.constants import COLOR_OFF, COLOR_BAD, COLORS

@click.command("set_leds", short_help="Set led colors")
@click.pass_context
@click.argument("data_key", required=True)
@click.argument("status_condition", required=True)
def set_leds(ctx, data_key, status_condition):
    """ This command will let you set the color of each led

    Examples:

    /b

    """
    data = {}
    
    if data_key in ctx.obj:
        data = ctx.obj[data_key]

    n_leds = len(utils.Leds)
    console.print("Setting " + status_condition + " for " + str(n_leds))

    bad_leds = []
    idx = 0

    for item in data:
        if idx >= 0 and idx < n_leds:
            if item.get_status_conditions()[status_condition]:
                color_to_use = COLORS[status_condition]
                utils.Leds[idx] = color_to_use
            else:
                utils.Leds[idx] = COLOR_BAD
                bad_leds.append(idx)
            console.print("Set led " + str(idx) + " to " + str(color_to_use))
        idx+= 1

    while idx < n_leds:
        utils.Leds[idx] = COLOR_OFF
        idx += 1

    utils.Leds.show()

    #blink bad nodes 5 times
    if len(bad_leds) > 0:
        for bad_led in bad_leds:
            fiveTimes = range(4)
            for blink in fiveTimes:
                if blink % 2 == 0:
                    utils.Leds[bad_led] = COLOR_BAD
                else:
                    utils.Leds[bad_led] = COLOR_OFF
                utils.Leds.show()    
                time.sleep(0.5)

    time.sleep(10)

    console.print("set_leds completed")

def random_color():
    return random.randrange(0, 7) * 32


import click, json
from click.core import Context
from kubernetes.client.api.core_v1_api import CoreV1Api
from kubeleds import console
from kubernetes import client
from kubernetes.client.rest import ApiException

@click.command("get_namespaced_pods", short_help="Login to a cluster and get pod data for a given namespace")
@click.pass_context
@click.argument("api_host", required=True, type=click.STRING)
@click.argument("api_token", required=True, type=click.STRING)
@click.argument("namespace", required=True, type=click.STRING)
@click.argument("verify_ssl", required=False, default=True, type=click.BOOL)
@click.argument("data_key", required=False, default="get_namespaced_pods", type=click.STRING)
def get_namespaced_pods(ctx, api_host, api_token, namespace, verify_ssl, data_key):
    """This command will let you login in to a get the status of pods in the passed namespace

    Examples:

    /b
        
    """
    console.print("api_host:" + api_host)
    console.print("api_token:" + api_token)
    console.print("namespace:" + namespace)
    #console.print("verify_ssl:" + str(verify_ssl))

    aToken = api_token
    aConfiguration= client.Configuration()
    aConfiguration.debug = True
    aConfiguration.host = api_host
    aConfiguration.verify_ssl = verify_ssl
    aConfiguration.api_key = {"authorization": "Bearer " + aToken}

    with client.ApiClient(aConfiguration) as aApiClient:
        v1 = client.CoreV1Api(aApiClient)

        pod_statuses = []

        try:
            
            response = v1.list_namespaced_pod(namespace=namespace, _preload_content=False, watch=False)
            #response = v1.list_namespaced_pod(namespace=namespace, _preload_content=False, watch=False)
            pods = json.loads(response.data)

            with open('pods.json', 'w') as outfile:
                json.dump(pods["items"], outfile)

            console.print("Pods Count: " + str(len(pods["items"])))
            
            for pod in pods["items"]:
                #console.print(node.metadata.name)
                pod_status = PodStatus(pod)
                #json_status = json.dumps(node_status.__dict__)
                #node_statuses[node_status.get_name()] = node_status
                pod_statuses.append(pod_status)

            ctx.obj[data_key] = pod_statuses
        except ApiException as e:
            console.print("Kubernetes error: %s\n" % e)

        return pod_statuses

class PodStatus:
    def __init__(self, pod_info):
        self.set_name(pod_info["metadata"]["name"])
        self.set_status_conditions(pod_info)

    def __getitem__(self, item):
        console.print(item)
        return getattr(self, item)

    def get_name(self):
        return self._name
    
    def get_status_conditions(self):
        return self._status_conditions
    
    def set_name(self, value):
        self._name = value

    def set_status_conditions(self, pod):
        self._status_conditions = {}
        for cond in pod["status"]["conditions"]:
            #console.print(cond)
            cond_result = False
            if cond["type"] == "Ready":
                if cond["status"] == "True":
                    cond_result = True
            else:
                if cond["status"] == "False":
                    cond_result = True

            self._status_conditions[cond["type"]] = cond_result

# @cluster.resultcallback()
# def process_commands(processors):
#     stream = ()

#     for processor in processors:
#         stream = processor(stream)

#     for _ in stream:
#         pass


# def processor(f):
#     def new_func(*args, **kwargs):
#         def processor(stream):
#             return f(stream, *args, **kwargs)

#         return processor
    
#     return update_wrapper(new_func, f)

# def generator(f):
#     @processor
#     def new_func(stream, *args, **kwargs):
#         yield from stream
#         yield from f(*args, **kwargs)

#     return update_wrapper(new_func, f)



"""
Exports for CLI commands.
"""
from kubeleds.commands.init import init
from kubeleds.commands.show import show
from kubeleds.commands.cluster import get_cluster_nodes
from kubeleds.commands.led import set_leds
from kubeleds.commands.pod import get_namespaced_pods